"""
AWS Django settings for eox_tagging project.

Juniper release will remove aws.py file. https://openedx.atlassian.net/browse/DEPR-14
"""
from __future__ import unicode_literals

from .production import plugin_settings  # pylint: disable=unused-import



"""
Serializers for tags and related objects.
"""
import re

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from rest_framework import serializers

from eox_tagging.api.v1 import fields
from eox_tagging.constants import AccessLevel, Status
from eox_tagging.edxapp_accessors import get_object, get_site
from eox_tagging.models import Tag

PROXY_MODEL_NAME = "opaquekeyproxymodel"
MODELS_WITH_COMPOUND_KEYS = {
    "courseenrollment": ["username", "course_id"],  # Compound keys
}


class TagSerializer(serializers.ModelSerializer):
    """Serializer for tag objects."""

    target_id = serializers.CharField(source='target_object', write_only=True)
    owner_id = serializers.CharField(source='owner_object', required=False, write_only=True)
    owner_type = serializers.CharField(source='owner_object_type', write_only=True, required=False)
    target_type = serializers.CharField(source='target_object_type', write_only=True)
    meta = serializers.SerializerMethodField()
    access = fields.EnumField(enum=AccessLevel)
    status = fields.EnumField(enum=Status, required=False)

    class Meta:  # pylint: disable=old-style-class, useless-suppression
        """Meta class."""
        model = Tag
        fields = ('meta', 'key', 'tag_value', 'tag_type', 'access', 'activation_date', 'expiration_date',
                  'target_id', 'owner_id', 'owner_type', 'target_type', 'status')

    def get_meta(self, instance):
        """Getter of read-only field that returns technical information."""
        return {
            "target_id": str(instance.target_object),
            "target_type": instance.target_object_type,
            "owner_id": str(instance.owner_object),
            "owner_type": instance.owner_object_type,
            "created_at": instance.created_at,
            "inactivated_at": instance.inactivated_at,
        }

    # Validation and creation of tags
    def create(self, validated_data):
        """Function that creates a Tag instance."""

        # Finding target and owner objects
        target_object = None
        owner_object = None
        target_type = validated_data.pop("target_object_type")
        owner_type = validated_data.pop("owner_object_type", None)
        target = validated_data.pop("target_object", None)

        if target_type in MODELS_WITH_COMPOUND_KEYS:
            data = self.__convert_compound_keys(target, target_type)
        else:
            data = {
                "target_id": target,
            }

        try:
            target_object = get_object(target_type, **data)
        except Exception:  # pylint: disable=broad-except
            serializers.ValidationError({"Target": _("Error getting {} object."
                                         .format(target_type))})

        if owner_type == "user":
            owner_object = self.context.get("request").user
        else:
            owner_object = get_site()

        # Set objects
        tag_object = {
            "target_object": target_object,
            "owner_object": owner_object,
        }
        tag_object.update(validated_data)

        try:
            return Tag.objects.create_tag(**tag_object)
        except ValidationError as e:
            raise serializers.ValidationError({"Tag": _("{}".format(e.message))})

    def __convert_compound_keys(self, ids, object_type):
        """
        Function that converts strings with format: `key1: key2` into a dictionary.
        """
        target_id = re.split(r':\s', ids)
        target_labels = MODELS_WITH_COMPOUND_KEYS.get(object_type)
        target_pairs = zip(target_labels, target_id)
        return dict(target_pairs)



"""Filter module for tags."""
from django.contrib.contenttypes.models import ContentType
from django_filters import rest_framework as filters

from eox_tagging.constants import AccessLevel
from eox_tagging.models import Tag

PROXY_MODEL_NAME = "opaquekeyproxymodel"


class TagFilter(filters.FilterSet):
    """Filter class for tags."""

    course_id = filters.CharFilter(name="course_id", method="filter_by_target_object")
    username = filters.CharFilter(name="username", method="filter_by_target_object")
    enrolled = filters.CharFilter(name="enrollment", method="filter_by_target_object")
    enrollments = filters.CharFilter(method="filter_enrollments")
    target_type = filters.CharFilter(method="filter_target_types")
    created_at = filters.DateTimeFromToRangeFilter(name="created_at")
    activation_date = filters.DateTimeFromToRangeFilter(name="activation_date")
    access = filters.CharFilter(method="filter_access_type")

    class Meta:  # pylint: disable=old-style-class
        """Meta class."""
        model = Tag
        fields = ['key', 'created_at', 'activation_date', 'status', 'course_id', 'enrolled', 'enrollments', 'username']

    def filter_by_target_object(self, queryset, name, value):
        """Filter that returns the tags associated with target."""
        TARGET_TYPES = {
            "course_id": "courseoverview",
            "username": "user",
            "enrollment": "courseenrollment",
        }
        if value:
            TARGET_IDENTIFICATION = {
                "enrollment": {
                    "username": self.request.user.username,
                    "course_id": str(value),
                },
            }
            DEFAULT = {
                name: str(value),
            }
            try:
                filter_params = {
                    "target_type": TARGET_TYPES.get(name),
                    "target_id": TARGET_IDENTIFICATION.get(name, DEFAULT),
                }
                queryset = queryset.find_all_tags_for(**filter_params)
            except Exception:  # pylint: disable=broad-except
                return queryset.none()

        return queryset

    def filter_enrollments(self, queryset, name, value):  # pylint: disable=unused-argument
        """Filter that returns the tags associated with the enrollments owned by the request user."""
        if value:
            try:
                ctype = ContentType.objects.get(model="courseenrollment")
                enrollments_queryset = queryset.find_all_tags_by_type("courseenrollment")

                query_enrollments = []
                for tag_enrollment in enrollments_queryset:
                    target_id = tag_enrollment.target_object_id
                    enrollment = ctype.get_object_for_this_type(id=target_id)
                    if str(enrollment.course_id) == str(value):
                        query_enrollments.append(tag_enrollment)

                return query_enrollments
            except Exception:  # pylint: disable=broad-except
                return queryset.none()

        return queryset

    def filter_target_types(self, queryset, name, value):  # pylint: disable=unused-argument
        """Filter that returns targets by their type."""
        if value:
            try:
                queryset = queryset.find_all_tags_by_type(str(value))
            except Exception:  # pylint: disable=broad-except
                return queryset.none()

        return queryset

    def filter_access_type(self, queryset, name, value):  # pylint: disable=unused-argument
        """Filters targets by their access type."""
        if value:
            value_map = {v.lower(): k for k, v in AccessLevel.choices()}
            access = value_map.get(value.lower())
            queryset = queryset.filter(access=access) if access else queryset.none()

        return queryset
